ODT_IMAGE = (
    '<draw:frame draw:name="img1" svg:width="{0}" svg:height="{1}" text:anchor-type="paragraph">'
    '<draw:image xlink:type="simple" xlink:show="embed" xlink:actuate="onLoad">'
    '<office:binary-data>{2}</office:binary-data>'
    '</draw:image></draw:frame>'
)
